class_name DragonInstantiator

var _dragon_template: PackedScene
var _viewport: Viewport
var _window: Window

func _init(dragon: PackedScene, viewport: Viewport, window: Window) -> void:
	_window = window
	_viewport = viewport
	_dragon_template = dragon


func instantiate(position: Vector2, hat: Texture2D, shirt: Texture2D, shoes: Texture2D, drag: bool) -> Node:
	if drag == false:
		return _instantiate_dragon(position, hat, shirt, shoes, drag)
	var relative_position: Vector2i = _calculate_relative_position(position)
	return _instantiate_dragon(relative_position, hat, shirt, shoes, drag)


func _calculate_window_scale() -> Vector2:
	var viewport_size: Vector2 = _viewport.get_texture().get_size()
	var window_size: Vector2 = _window.get_size_with_decorations()
	
	return window_size / viewport_size


func _calculate_relative_position(position: Vector2) -> Vector2i:
	var scale: Vector2 = _calculate_window_scale()
	return Vector2i(Vector2(position) * scale)


func _instantiate_dragon(relative_position: Vector2i, hat: Texture2D, shirt: Texture2D, shoes: Texture2D, drag: bool) -> Node:
	var dragon: Dragon = _dragon_template.instantiate()
	
	var window_position: Vector2i = DisplayServer.window_get_position()
	var window_size: Vector2i = DisplayServer.window_get_size()
	
	if drag == false:
		dragon.position = relative_position
	else:
		dragon.position = window_position + relative_position - dragon.size / 2
	dragon.dress(hat, shirt, shoes)
	
	var size: float = _calculate_window_scale().y
	dragon.main_window_rect = Rect2i(window_position, window_size)
	dragon.show()
	
	if drag:
		dragon.start_dragon_drag()
	
	return dragon
